<?php
/**
 * 2007-2021 PrestaShop
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 *
 *
 * @author    yourName
 * @copyright 2007-2021
 * @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 *
 * @generated by PrestashopConsole  / copyright 2016-2021 Hennes Hervé http://www.h-hennes.fr/blog/
 */

if (!defined('_PS_VERSION_')) {
    exit;
}


class Hh_demoemails extends Module
{

    /**
     * @var string[] Listes des hooks prestashop liés aux emails
     */
    protected $_hooks = [
        'actionEmailSendBefore',
        'sendMailAlterTemplateVars',
        'actionEmailAddBeforeContent',
        'actionEmailAddAfterContent',
        'actionGetExtraMailTemplateVars',
        'actionMailAlterMessageBeforeSend',
    ];

    public function __construct()
    {
        $this->name = 'hh_demoemails';
        $this->tab = 'others';
        $this->version = '0.1.0';
        $this->author = 'hhennes';
        $this->bootstrap = true;
        parent::__construct();

        $this->displayName = $this->l('Hh demo email');
        $this->description = $this->l('Show module email possibilites');
    }

    /**
     * Installation du module
     * @return bool
     */
    public function install()
    {
        if (
            !parent::install()
            || !$this->registerHook($this->_hooks)
        ) {
            return false;
        }

        return true;
    }

    /**
     * Configuration du module
     */
    public function getContent()
    {
        //Ne fait rien sauf envoyer l'email de demo
        $this->sendDemoModuleEmail();
    }

    /**
     * Envoi d'un email spécifique au module
     */
    public function sendDemoModuleEmail()
    {
        //cf. documentation officielle https://devdocs.prestashop.com/1.7/development/mail/

        //Variable de l'email, par convention la clé est sous la forme {variable}
        $emailsVars = [
            '{variable1}' => 'Contenu variable 1',
            '{variable2}' => 'Contenu variable 2'
        ];

        Mail::send(
            $this->context->language->id, //Identifiant de langue
            'demo', //Code du template ( sans extension) Attention il faut créer un fichier html et txt de chaque template
            $this->l(' Module Installation', false, $this->context->language->locale), // Sujet de l'email ( on peut forcer la traduction dans une locale souhaitée)
            $emailsVars,//Variables de l'email
            'email@test.com', // Email de l'adresse
            NULL, //receiver name
            NULL, //from email address
            NULL,  //from name
            NULL, //file attachment
            NULL, //mode smtp
            _PS_MODULE_DIR_.'hh_demoemails/mails' //La ligne importante est ici ( chemin du dossier mail de votre module )
        );

    }

    /**
     * Excécuté AVANT l'envoi d'un emails
     * @param array $params [
     * 'idLang' => &$idLang,
     * 'template' => &$template,
     * 'subject' => &$subject,
     * 'templateVars' => &$templateVars,
     * 'to' => &$to,
     * 'toName' => &$toName,
     * 'from' => &$from,
     * 'fromName' => &$fromName,
     * 'fileAttachment' => &$fileAttachment,
     * 'mode_smtp' => &$mode_smtp,
     * 'templatePath' => &$templatePath,
     * 'die' => &$die,
     * 'idShop' => &$idShop,
     * 'bcc' => &$bcc,
     * 'replyTo' => &$replyTo,
     * ]
     */
    public function hookActionEmailSendBefore($params)
    {

        if ($params['template'] == 'contact') {

            if ($params['to'] == 'nosend@test.com') {
                //Sinon on ne veut pas envoyer d'email
                return false;
            }

            if ($params['to'] == 'contact@boutique.com' ) {

                $params['from'] = 'modified@test.com';
                $params['subject'] = 'Custom contact subject';
                $params['fromName'] = 'Test expéditeur';
                $params['template'] = 'contact_custom';
                $params['templatePath'] = _PS_MODULE_DIR_.$this->name . '/mails/';
                //Ajout d'une variable
                $params['templateVars']['{date_envoi}'] = date('Y-m-d H:i:s');
                //Ajout d'un email en bcc
                $params['bcc'] = ['test@yopmail.com'];
            }

        }

    }

    /**
     * Cette fonction permets de modifier spécifiques des variables de l'email
     * @param array $params
     *  [
     *    'template' => 'nom_du_template',
     *    'template_vars' => &[//tableau des variables de l'email]
     *  ]
     */
    public function hookSendMailAlterTemplateVars($params)
    {
        //Conditionnement du template
        if ($params['template'] == 'account') {
            //Pour connaitre les variables définies regarder l'email source.
            //Ou faire un dump( de $params['template_vars'] )
            //Dans notre exemple on veut remplacer le prénom de l'utilisateur par "Jean Michel" et son prénom par "A peu près"
            $params['template_vars']['{firstname}'] = 'Jean Michel';
            $params['template_vars']['{lastname}'] = ' A peu près';
        }
    }

    /**
     * Hook appellé AVANT la récupération du contenu du template de l'email
     * @param array $params [
     * 'template' => $template,
     * 'template_html' => &$templateHtml,
     * 'template_txt' => &$templateTxt,
     * 'id_lang' => (int) $idLang,
     * ]
     */
    public function hookActionEmailAddBeforeContent($params)
    {
        //Pas d'exemple parlant et utile pour ce hook nativement
        //Regarder https://www.h-hennes.fr/blog/2019/05/23/prestashop-simplification-de-la-mise-en-page-des-emails/ par ex
    }

    /**
     * Hook appellé APRES la récupération du contenu du template de l'email
     * @param array $params [
     * 'template' => $template,
     * 'template_html' => &$templateHtml,
     * 'template_txt' => &$templateTxt,
     * 'id_lang' => (int) $idLang,
     * ]
     */
    public function hookActionEmailAddAfterContent($params)
    {
        //Pour l'exemple ici on va supprimer la variable {shop_name} du contenu de l'email de contact
        if ($params['template'] == 'contact') {
            $params['template_html'] = str_replace('{shop_name}', '', $params['template_html']);
            $params['template_txt'] = str_replace('{shop_name}', '', $params['template_txt']);
        }
    }

    /**
     * Ce hook permets d'ajouter des nouvelles variables à un email existant
     * L'inconvénient est qu'il faut modifier le template source pour ajouter les placeholder de vos variables
     *
     * [
     * 'template' => $template,
     * 'template_vars' => $templateVars,
     * 'extra_template_vars' => &$extraTemplateVars,
     * 'id_lang' => (int) $idLang,
     * ],
     *
     * @param array $params
     */
    public function hookActionGetExtraMailTemplateVars($params)
    {
        //Ajout de variables personnalisées dans l'email de contact
        if ($params['template'] == 'contact') {
            //Les variables sont passés par références, pas besoin de passer de return
            $params['extra_template_vars']['{date_envoi}'] = date('Y-m-d H:i:s');
            $params['extra_template_vars']['{module_modification}'] = $this->name;
        }
    }

    /**
     * @param array $params
     * [
     * 'message' => &$message, //Instance de Swift_Message
     * ]
     */
    public function hookActionMailAlterMessageBeforeSend($params)
    {
        //Permets de jouer avec l'objet Swift_Message
        // De mon côté je n'ai pas eut l'utilisé de ce hook
    }


}
